/* complex bessel function */
/* Written by Emre Tuncer */
#include <math.h>
#include <stdio.h>
#include "../src/complex.h"


complex fo(complex x)
{
  complex t,f0;
  
  t=3./x;
  f0=0.79788456-.00000077*t;
  f0=sub(f0,mul(r2c(.00552740,0.),cpow(t,2.)));
  f0=sub(f0,mul(r2c(.00009512,0.),cpow(t,3.)));
  f0=add(f0,mul(r2c(.00137237,0.),cpow(t,4.)));  
  f0=sub(f0,mul(r2c(.00072805,0.),cpow(t,5.)));
  f0=add(f0,mul(r2c(.00014476,0.),cpow(t,6.)));
  return f0;
}

complex tho(complex x)
{
  complex t,t0;
  
  t=div_(r2c(3.,0.),x);
  t0=sub(x,r2c(.78539816,0.));
  t0=sub(t0,mul(r2c(.04166397,0.),t));
  t0=sub(t0,mul(r2c(.00003954,0.),cpow(t,2.)));
  t0=add(t0,mul(r2c(.00262573,0.),cpow(t,3.)));
  t0=sub(t0,mul(r2c(.00054125,0.),cpow(t,4.)));
  t0=sub(t0,mul(r2c(.00029333,0.),cpow(t,5.)));
  t0=add(t0,mul(r2c(.00013558,0.),cpow(t,6.)));
  return t0;
}


complex fi(complex x)
{
  complex t,f0;
  
  t=div_(r2c(3.,0.),x);
  f0=add(r2c(0.79788456,0.),mul(r2c(.00000156,0.),t));
  f0=add(f0,mul(r2c(.01659667,0.),cpow(t,2.)));
  f0=add(f0,mul(r2c(.00017105,0.),cpow(t,3.)));
  f0=sub(f0,mul(r2c(.00249511,0.),cpow(t,4.)));  
  f0=add(f0,mul(r2c(.00113653,0.),cpow(t,5.)));
  f0=sub(f0,mul(r2c(.00020033,0.),cpow(t,6.)));
  return f0;
}

complex thi(complex x)
{
  complex t,t0;
  
  t=div_(r2c(3.,0.),x);
  t0=sub(x,r2c(2.35619449,0.));
  t0=add(t0,mul(r2c(.12499612,0.),t));
  t0=add(t0,mul(r2c(.00005650,0.),cpow(t,2.)));
  t0=sub(t0,mul(r2c(.00637879,0.),cpow(t,3.)));
  t0=add(t0,mul(r2c(.00074348,0.),cpow(t,4.)));
  t0=add(t0,mul(r2c(.00079824,0.),cpow(t,5.)));
  t0=sub(t0,mul(r2c(.00029166,0.),cpow(t,6.)));
  return t0;
}

complex J0(complex x)
{
  complex t,t1;

  if (cabs_(x)<3.) {
    t=cpow(div_(x,r2c(3.,0.)),2.);
    t1=sub(r2c(1.,0.),mul(r2c(2.2499997,0.),t));
    t1=add(t1,mul(r2c(1.2656208,0.),mul(t,t)));
    t1=sub(t1,mul(r2c(.3163866,0.),cpow(t,3.)));
    t1=add(t1,mul(r2c(.0444479,0.),cpow(t,4.)));
    t1=sub(t1,mul(r2c(.0039444,0.),cpow(t,5.)));
    t1=add(t1,mul(r2c(.0002100,0.),cpow(t,6.)));
  }
  else 
    t1=mul(cpow(x,-0.5),mul(fo(x),ccos(tho(x))));
  
 return t1;
}


complex Y0(complex x)
{
  complex t,t1;

  if (cabs_(x)<3.) {
    t=cpow(div_(x,r2c(3.,0.)),2.);
    t1=mul(mul(r2c(2/M_PI,0.),Clog(mul(r2c(.5,0.),x))),J0(x));
    t1=add(t1,r2c(.36746691,.0));
    t1=add(t1,mul(r2c(.60559366,0.),cpow(t,1.)));
    t1=sub(t1,mul(r2c(.74350384,0.),cpow(t,2.)));
    t1=add(t1,mul(r2c(.25300117,0.),cpow(t,3.)));
    t1=sub(t1,mul(r2c(.04261214,0.),cpow(t,4.)));
    t1=add(t1,mul(r2c(.00427916,0.),cpow(t,5.)));
    t1=sub(t1,mul(r2c(.00024846,0.),cpow(t,6.)));
  }
  else 
    t1=mul(cpow(x,-.5),mul(fo(x),csin(tho(x))));
  
 return t1;
}

complex J1(complex x)
{
  complex t,t1;

  if (cabs_(x)<3.) {
    t=cpow(div_(x,r2c(3.,0.)),2.);
    t1=sub(r2c(.5,0.),mul(r2c(.56249985,0.),t));
    t1=add(t1,mul(r2c(.21093573,0.),mul(t,t)));
    t1=sub(t1,mul(r2c(.03954289,0.),cpow(t,3.)));
    t1=add(t1,mul(r2c(.00443319,0.),cpow(t,4.)));
    t1=sub(t1,mul(r2c(.00031761,0.),cpow(t,5.)));
    t1=add(t1,mul(r2c(.00001109,0.),cpow(t,6.)));
    t1=mul(t1,x);
  }
  else 
    t1=mul(cpow(x,-.5),mul(fi(x),ccos(thi(x))));
  
 return t1;
}


complex Y1(complex x)
{
  complex t,t1;

  if (cabs_(x)<3.) {
    t=cpow(div_(x,r2c(3.,0.)),2.);
    t1=mul(mul(r2c(2/M_PI,0.),Clog(mul(r2c(.5,0.),x))),J1(x));
    t1=mul(t1,x);
    t1=sub(t1,r2c(.6366198,.0));
    t1=add(t1,mul(r2c(.2212091,0.),cpow(t,1.)));
    t1=add(t1,mul(r2c(2.1682709,0.),cpow(t,2.)));
    t1=sub(t1,mul(r2c(1.3164827,0.),cpow(t,3.)));
    t1=add(t1,mul(r2c(.3123951,0.),cpow(t,4.)));
    t1=sub(t1,mul(r2c(.0400976,0.),cpow(t,5.)));
    t1=add(t1,mul(r2c(.0027873,0.),cpow(t,6.)));
    t1=div_(t1,x);
  }
  else 
    t1=mul(cpow(x,-.5),mul(fi(x),csin(thi(x))));
  
 return t1;
}

complex H0(int i,complex x)
{
  complex j=r2c(0.,1.);
  if (cabs_(x) < 20.) {
    if (i==1) return add(J0(x),mul(r2c(0.,1.),Y0(x)));
    x=r2c(RE(x),-IM(x));
    return conj(add(J0(x),mul(r2c(0.,1.),Y0(x))));
  }
  else {
    if (i==1)  return cpow((2./M_PI/x),0.5)*cexp(j*(x-M_PI/4.));
    else return cpow((2./M_PI/x),0.5)*cexp(j*(-1*(x-M_PI/4.)));
  }
}

complex H1(int i,complex x)
{
  complex j=r2c(0.,1.);
  if (cabs_(x) < 20.) {
    if (i==1) return add(J1(x),mul(r2c(0.,1.),Y1(x)));
    x=r2c(RE(x),-IM(x));
    return conj(add(J1(x),mul(r2c(0.,1.),Y1(x))));
  }
  else {
    if (i==1) return cpow((2./M_PI/x),0.5)*cexp(j*(x-M_PI/4.-M_PI/2.));
    else return cpow((2./M_PI/x),0.5)*cexp(-1*j*(x-M_PI/4.-M_PI/2.));
  }
}



