/* EII based on transmission line */

#include "../include/bessel.h"
#include <stdio.h>


complex Zsbeom(double w,double thick,double width,
	double wdum,double height,double
	nseg,int jj, int index,double si);

complex Zscir(double w,double r,double si);
complex Zstrap(double w,double si,double x1,double x2,double y1,double y2);

complex Zstriangle(double w,double W,double H,double si)
{
  complex t1,t2;
  complex J=r2c(0.,1.);
  static double mu=1.257e-6;
  t1=cpow(r2c(-1,0)*J*w*mu*si,0.5);
  t2=t1*J0(t1*H)/J1(t1*H)/si/W;
  return t2;
}


complex Zstrap(double w,double si,double x1,double x2,double y1,double y2,int ind)
{
  complex r1,r2,t;
  double w2,w1,h;
  complex J=r2c(0.,1.);
  double mu=1.257e-6;
  complex Z;

  if (ind==4) {
    if (fabs(x2-x1) > fabs(y2-y1)) {
      w2=fabs(x2-x1);
      w1=w2-fabs(y2-y1);
      h=fabs(y2-y1)/2.;
      t=J*cpow(J*w*mu*si,0.5);
      r1=t*h*w1/(w2-w1); r2=t*h*w2/(w2-w1);
      Z=r2c(-1,0)*J*cpow(J*w*mu/si,0.5)*(H1(1,r1)*H0(2,r2)-H1(2,r1)*H0(1,r2))/
	(H1(1,r2)*H1(2,r1)-H1(1,r1)*H1(2,r2))/w2;
      return Z;
    }
    else {
      w2=fabs(x2-x1);
      return Zstriangle(w,w2,w2/2.,si);
    }
  }
  else {
    if (fabs(x2-x1) < fabs(y2-y1)) {
      w2=fabs(y2-y1);
      w1=w2-fabs(x2-x1);
      h=fabs(x2-x1)/2.;
      t=J*cpow(J*w*mu*si,0.5);
      r1=t*h*w1/(w2-w1); r2=t*h*w2/(w2-w1);
      Z=r2c(-1,0)*J*cpow(J*w*mu/si,0.5)*(H1(1,r1)*H0(2,r2)-H1(2,r1)*H0(1,r2))/
	(H1(1,r2)*H1(2,r1)-H1(1,r1)*H1(2,r2))/w2;
      return Z;
    }
    else {
      w2=fabs(y2-y1);
      return Zstriangle(w,w2,w2/2.,si);
    }
  }
}



complex Zsbeom(double w,double x1,double y1,double x2,double y2,double x,double si)
{
  double pi,wo,wwo,ho,lo;
  double delta,mu,Dw;
  int kl,index;
  complex Z1;

  pi = M_PI;
  mu = 1.257e-6;
  delta = 1./sqrt(w*.5*mu*si);

  double wide = (x2-x1)/2.;
  double thick = (y2-y1)/2.;
  double tem = x-x1;
  if( wide > thick ) Dw = thick;
  else Dw = wide;
  if( tem > wide ) tem = 2.*wide-tem;
  if(3.*delta >= Dw) {
    if( tem > Dw ) {
      ho = Dw; 
      wwo = 0.;
      index = 2;
      Z1 = Zsbeom(w,Dw,2.*wide-2.*Dw,wwo,ho,4,kl,index,si);
    }
    else
    {
      kl = (int)((Dw-tem)/Dw*4.+1.);
      wo = Dw;
      wwo = 0.0;  
      ho = Dw;
      index = 0;
      Z1 = Zsbeom(w,Dw,wo,wwo,ho,4,kl,index,si);
    }
  } 
  else 
  {
    if( 3.*delta < Dw ) {
      if(tem < 3.*delta) {
        kl = (int)((3.*delta-tem)/3/delta*4.+1.);
        wo = 3.*delta;
        wwo = 0;
        ho = 3.*delta;
        index = 0;
        Z1 = Zsbeom(w,3.*delta,wo,wwo,ho,4,kl,index,si);
      }
      else {
        ho = 3.*delta; 
        wwo = 0.;
        index = 3;
        Z1 = Zsbeom(w,3.*delta,2.*wide-6.*delta,wwo,ho,4,kl,index,si);
      }
    }
  }
  return Z1;
}

complex Zsbeom(double w,double thick,double width,
                    double wdum,double height,double 
                    nseg,int jj, int index,double si)
{
  complex B,Ba;
  complex Z,Y,Zl,Zm;
  double mu,xjj;
  double hn,wn,tem1,tem2,tem3,tem4;

  xjj = (double)jj;
  mu = 4.*M_PI*1e-7;
  B = cpow(r2c(0.,w*mu*si),.5);
  Y = r2c(si,0.);

  if( w < .1 ) w = 1.;
  if( index == 0 ) 
  {
    hn = sqrt(pow(height,2)+pow((.5+xjj)/nseg*width+wdum,2));
    tem1 = hn*width/nseg/height/2.;
    tem2 = ((1.+xjj)*width/nseg+wdum)/height;
    tem3 = ((.5+xjj)*width/nseg+wdum)/height;
    tem4 = (xjj*width/nseg+wdum)/height;
    wn = tem1*(1./(1.+tem2*tem3)+1./(1.+tem3*tem4));
  
    Ba = r2c(0.,hn)*B;

    Z = r2c(0.,1.)*B/Y*J0(Ba)/J1(Ba)*r2c(width/nseg/wn,0.);
    return Z;
  }
  if( index == 2)
  {
    Z = B/Y*ccosh(B*height)/csinh(B*height);
    return Z;
  }
  if( index == 3)
  {
    Z = B/Y*csinh(B*height)/ccosh(B*height);
    return Z;
  }
  return r2c(1.e-12,1.e-12);
}

complex Zscir(double w,double r,double si)
{
  double  mu=4*M_PI*1e-7;
  complex tmp1=r2c(0,1)*cpow((r2c(0,1)*w*mu/si),0.5);
  complex tmp2=r2c(0,1)*r*cpow((r2c(0,1)*w*mu*si),0.5);
  return tmp1*J0(tmp2)/J1(tmp2);
}

complex Zsplt(double w,double t,double si)
{
  double  mu=4*M_PI*1e-7;
  complex tmp1=cpow((r2c(0,1)*w*mu/si),0.5);
  complex tmp2=cpow((r2c(0,1)*w*mu*si),0.5);
  return tmp1*ccosh(tmp2*t)/csinh(tmp2*t);
}
