/* complex number routine */

#include <math.h>
#include <errno.h>
#include "complex.h"

void complex::eq(double a,double b)
{
  x=a; y=b;
}


double cabs_(complex z)
{
  return sqrt(z.x*z.x+z.y*z.y);
}

double mag(double a,double b)
{
  return sqrt(a*a+b*b);
}


complex conj(complex a)
{
  a.y=-a.y;
  return a;
}

complex neg(complex a)
{
  a.x=-a.x;
  a.y=-a.y;
  return a;
}

complex mul(complex a,complex b)
{
  return a*b;
}

complex div_(complex a,complex b)
{
  return a/b;
}

complex add(complex a,complex b)
{
  return a+b;
}

complex sub(complex a,complex b)
{
  return a-b;
}

complex operator* (complex a,complex b)
{
  complex c;

  c.x=a.x*b.x-a.y*b.y;
  c.y=a.x*b.y+a.y*b.x;
  return c;
}

complex operator* (complex a,double b)
{
  return r2c(b*a.x,b*a.y);
}

complex operator* (double a,complex b)
{
  return r2c(a*b.x,a*b.y);
}

complex operator/ (complex a,complex b)
{
  complex c;
  double d;

//  if ((d=cabs_(b))==0.0) {
//    c.x=HUGE_VAL;
//    c.y=0;
//    errno=ERANGE;
//  }
//  else {
    d=b.x*b.x+b.y*b.y;
    c=a*conj(b);
    c.x/=d;
    c.y/=d;
//  }
  return c;
}

complex operator/ (complex a,double b)
{
  return a/r2c(b,0.);
}

complex operator/ (double a,complex b)
{
  return r2c(a,0.)/b;
}

complex operator+ (complex a,complex b)
{
  a.x+=b.x;
  a.y+=b.y;
  return a;
}

complex operator+ (complex a,double b)
{
  return r2c(a.x+b,a.y);
}

complex operator+ (double a,complex b)
{
  return r2c(a+b.x,b.y);
}

complex operator- (complex a,complex b)
{
  a.x-=b.x;
  a.y-=b.y;
  return a;
}

complex operator- (complex a,double b)
{
  return r2c(a.x-b,a.y);
}

complex operator- (double a,complex b)
{
  return r2c(a-b.x,-b.y);
}


double arg(complex a)
{
  if (cabs_(a)==0) return 0.0;
  return atan2(a.y,a.x);
}


complex cexp(complex a)
{
  complex b;

  b.x=exp(a.x);
  b.y=b.x*sin(a.y);
  b.x*=cos(a.y);
  return b;
}

complex Clog(complex a)
{
  complex b;

  b.x=log(cabs_(a));
  b.y=arg(a);
  return b;
}

complex csinh(complex a)
{
  complex b;

  b.eq(2.0,0.);
  return (cexp(a)-cexp(neg(a)))/b;
}

complex ccosh(complex a)
{
  complex b;

  b.eq(2.,0.);
  return (cexp(a)+cexp(neg(a)))/b;
}

complex ctanh(complex a)
{
  if (a.x>80.) return r2c(1.,0.);
  if (a.x<-80.) return r2c(-1.,0.);
  return csinh(a)/ccosh(a);
}


complex csin(complex a)
{
  complex b,j;

  b.eq(0.,2.);
  j.eq(0.,1.);
  return (cexp(j*a)-cexp(neg(j*a)))/b;
}

complex ccos(complex a)
{
  complex b,j;

  b.x=2.0;
  b.y=0.0;
  j=r2c(0.,1.);
  return (cexp(j*a)+cexp(neg(j*a)))/b;
}



complex cpow(complex x,double y)
{
   double a,b;
   complex c;

   a=y*arg(x);
   b=pow(cabs_(x),y);
   c.x=b*cos(a);
   c.y=b*sin(a);

   return c;
}


complex r2c(double a,double c)
{
  complex b;

  b.x=a;
  b.y=c;
  return b;
}

double RE(complex a)
{
  return a.x;
}

double IM(complex a)
{
  return a.y;
}







