/* Mutual inductance calculation between ribbons */
/* for circular conductor I used numerical integral while */
/* closed form equation is used for rectangular conductor */
/* Sangwoo Kim 07/14/98 */

#include "weeks.h"
#include "inductance.h"
#include "integral.h"
#include <stdio.h>

extern Sys *sis;

static double xs,ys;
static double (*nrfunc)(double,double);
double yy1,yy2,tha1,tha2,X1,X1p,Y1,Y1p;

double FF(double X,double Xp)
{
  double tmp;
  tmp=(X*cos(tha1)-Xp*cos(tha2)+X1-X1p)*(X*cos(tha1)-Xp*cos(tha2)+X1-X1p)
	+(X*sin(tha1)-Xp*sin(tha2)+Y1-Y1p)*(X*sin(tha1)-Xp*sin(tha2)+Y1-Y1p);
  if (tmp <= 0.) tmp=1e-10;
  return log(tmp);
}


double int2d(double (*func)(double,double),double x1,double x2)
{
  double f1(double xx);
  nrfunc=func;
  return gauss(f1,x1,x2);
}

double f1(double xx)
{
  double f2(double yy);
  xs=xx;
  return gauss(f2,yy1,yy2);
}


double f2(double yy)
{
  return (*nrfunc)(xs,yy);
}

double F(double y,double z)
{
  double a,b;

  if (y==0 && z==0) return 0.;
  if (y==0 || z==0) {
    y+=z;
    return pow(y,4.)/24*log(y*y);
  }
  a=(pow(y,4.)-6*y*y*z*z+pow(z,4.))/24.*log(y*y+z*z);
  a-=y*z/3.*(y*y*atan(z/y)+z*z*atan(y/z));
  return a;
}



double Lp(Filament* a,Filament* b)
{
  double y1,y2,z2,z1,yp1,yp2,zp2,zp1;
  double sum,mu,x,y,m;
  double wid1,wid2;
  double x1p,x2p,y1p,y2p;
  double A1,A2,der;
  
  mu=4e-7*M_PI;
  y1=a->x1; y2=a->x2; z1=a->y1; z2=a->y2;
  yp1=b->x1; yp2=b->x2; zp1=b->y1; zp2=b->y2;
  der=sqrt((y1-yp1)*(y1-yp1)+(z1-zp1)*(z1-zp1));
  wid1=sqrt((y2-y1)*(y2-y1)+(z2-z1)*(z2-z1));
  wid2=sqrt((yp2-yp1)*(yp2-yp1)+(zp2-zp1)*(zp2-zp1));

  if (a->pos==20 || b->pos==20) {
    A1=(y1-y2)*(z1-z2);
    A2=(yp1-yp2)*(zp1-zp2);
    sum=  F(y2-yp2,z2-zp2)-F(y1-yp2,z2-zp2)-F(y2-yp1,z2-zp2)+F(y1-yp1,z2-zp2);
    sum+=-F(y2-yp2,z1-zp2)+F(y1-yp2,z1-zp2)+F(y2-yp1,z1-zp2)-F(y1-yp1,z1-zp2);
    sum+=-F(y2-yp2,z2-zp1)+F(y1-yp2,z2-zp1)+F(y2-yp1,z2-zp1)-F(y1-yp1,z2-zp1);
    sum+= F(y2-yp2,z1-zp1)-F(y1-yp2,z1-zp1)-F(y2-yp1,z1-zp1)+F(y1-yp1,z1-zp1);
    sum+=25./6.*(y2-y1)*(yp2-yp1)*(z2-z1)*(zp2-zp1);
    sum*=mu/4/M_PI/A1/A2;
    return sum;
  }

  if (der > 50*wid1 || der > 50*wid2 || a->parent->lp==2 || b->parent->lp==2) {
    tha1=atan((z2-z1)/(y2-y1));
    tha2=atan((zp2-zp1)/(yp2-yp1));

    yy1=0; yy2=wid2;
    X1=y1; X1p=yp1; Y1=z1; Y1p=zp1;
    sum=-int2d(FF,wid1*1e-4,wid1)*mu/(4*M_PI*wid1*wid2);
    return sum;
  }
  else {
    A1=(y1-y2)*(z1-z2);
    A2=(yp1-yp2)*(zp1-zp2);
    sum=  F(y2-yp2,z2-zp2)-F(y1-yp2,z2-zp2)-F(y2-yp1,z2-zp2)+F(y1-yp1,z2-zp2);
    sum+=-F(y2-yp2,z1-zp2)+F(y1-yp2,z1-zp2)+F(y2-yp1,z1-zp2)-F(y1-yp1,z1-zp2);
    sum+=-F(y2-yp2,z2-zp1)+F(y1-yp2,z2-zp1)+F(y2-yp1,z2-zp1)-F(y1-yp1,z2-zp1);
    sum+= F(y2-yp2,z1-zp1)-F(y1-yp2,z1-zp1)-F(y2-yp1,z1-zp1)+F(y1-yp1,z1-zp1);
    sum+=25./6.*(y2-y1)*(yp2-yp1)*(z2-z1)*(zp2-zp1);
    sum*=mu/4/M_PI/A1/A2;
    return sum;
  }
}
